@@ -32,6 +32,7 @@ stack::call::call() : TimerQueue::event(Timer::reset), segments()
 	enlist(&stack::sip);
 	starting = ending = 0l;
 	reason = joined = NULL;
+	rtp = NULL;
 }
 
 void stack::call::terminateLocked(void)
@@ -60,7 +61,9 @@ void stack::call::joinLocked(session *join)
 				s->reg->decUse();
 				s->reg = NULL;
 			}
-			if(s->cid > 0 && s->state != session::CLOSED) {
+			if(s->state == session::REFER)
+				s->state = session::CLOSED;
+			else if(s->cid > 0 && s->state != session::CLOSED) {
 				s->state = session::CLOSED;
 				eXosip_lock();
 				eXosip_call_terminate(s->cid, s->did);
@@ -101,6 +104,12 @@ void stack::call::disconnectLocked(void)
 	case TERMINATE:
 		reason = "terminated";
 		break;
+	case TRANSFER:
+		reason = "transfer";
+		break;
+	case REDIRECT:
+		reason = "redirect";
+		break;
 	case INITIAL:
 	case FINAL:
 		break;
@@ -112,7 +121,9 @@ void stack::call::disconnectLocked(void)
 			sp->sid.reg->decUse();
 			sp->sid.reg = NULL;
 		}
-		if(sp->sid.cid > 0 && sp->sid.state != session::CLOSED) {
+		if(sp->sid.state == session::REFER)
+			sp->sid.state = session::CLOSED;
+		else if(sp->sid.cid > 0 && sp->sid.state != session::CLOSED) {
 			sp->sid.state = session::CLOSED;
 			eXosip_lock();
 			eXosip_call_terminate(sp->sid.cid, sp->sid.did);
@@ -154,10 +165,13 @@ void stack::call::reply_source(int error)
 	eXosip_unlock();
 }
 
-void stack::call::bye(session *s)
+void stack::call::bye(thread *thread, session *s)
 {
 	bool closing = false;
+
 	Mutex::protect(this);
+	s->tid = 0;	// cleared already....
+
 	switch(state) {
 	case JOINED:
 	case ANSWERED:
@@ -172,7 +186,7 @@ void stack::call::bye(session *s)
 	case RINGBACK:
 		if(s == source) {
 			s->state = session::CLOSED;
-			terminateLocked();
+			disconnectLocked();
 		}
 		else
 			closing = true;
@@ -232,6 +246,8 @@ void stack::call::failed(thread *thread, session *s)
 		--ringing;
 	else if(s->state == session::BUSY)
 		--ringbusy;
+	else if(s->state == session::REFER)
+		s->state = session::CLOSED;
 	if(s->state != session::CLOSED)
 		s->state = session::OPEN;
 
@@ -261,6 +277,100 @@ void stack::call::failed(thread *thread, session *s)
 	stack::close(s);
 }
 
+void stack::call::reinvite(thread *thread, session *s)
+{
+	osip_message_t *reply = NULL;
+	osip_body_t *body = NULL;
+	int did;
+
+	assert(thread != NULL);
+	assert(s != NULL);
+
+	Mutex::protect(this);
+	s->did = thread->sevent->did;
+	if(s == source) {
+		if(target)
+			did = target->did;
+		else
+			goto unconnected;
+	}
+	else
+		did = source->did;
+
+	s->tid = thread->sevent->tid;
+	body = NULL;
+	osip_message_get_body(thread->sevent->request, 0, &body);
+	if(body && body->body)
+		String::set(s->sdp, sizeof(s->sdp), body->body);
+
+	switch(state) {
+	case RINGING:
+	case RINGBACK:
+	case TRYING:
+	case ANSWERED:
+		if(s == source) {
+unconnected:
+			Mutex::release(this);
+			eXosip_lock();
+			eXosip_call_build_answer(source->tid, 500, &reply);
+			if(reply != NULL) {
+				osip_message_set_header(reply, "Reply-After", "8");
+				stack::siplog(reply);
+				eXosip_call_send_answer(source->tid, 500, reply);
+			}
+			eXosip_unlock();
+			return;
+		}
+		if(state != ANSWERED && state != RINGBACK) {
+			joinLocked(s);
+			if(state != ANSWERED)
+				state = RINGBACK;
+		}
+		time(&expires);
+		expires += thread->header_expires;
+		arm((timeout_t)(thread->header_expires * 1000l));
+		Mutex::release(this);
+		eXosip_lock();
+		eXosip_call_build_answer(source->tid, 200, &reply);
+		if(reply != NULL) {
+			osip_message_set_require(reply, "100rel");
+			osip_message_set_header(reply, "RSeq", "1");
+			if(body && body->body) {
+				osip_message_set_body(reply, body->body, strlen(body->body));
+				osip_message_set_content_type(reply, "application/sdp");
+			}
+			stack::siplog(reply);
+			eXosip_call_send_answer(source->tid, 200, reply);
+		}
+		eXosip_unlock();
+		return;
+	case HOLDING:
+	case JOINED:
+		state = JOINED;
+		time(&expires);
+		expires += thread->header_expires;
+		arm((timeout_t)(thread->header_expires * 1000l));
+		Mutex::release(this);
+		eXosip_lock();
+		eXosip_call_build_request(did, "INVITE", &reply);
+		if(reply != NULL) {
+			osip_message_set_supported(reply, "100rel,replaces");
+			if(body && body->body) {
+				osip_message_set_body(reply, body->body, strlen(body->body));
+				osip_message_set_content_type(reply, "application/sdp");
+			}
+			stack::siplog(reply);
+			eXosip_call_send_request(did, reply);
+		}
+		eXosip_unlock();
+		return;
+	}
+	Mutex::release(this);
+	debug(2, "reinvite failed for call %08x:%u",
+			source->sequence, source->cid);
+		failed(thread, s);
+}
+		
 void stack::call::answer(thread *thread, session *s)
 {
 	osip_message_t *reply = NULL;
@@ -326,6 +436,56 @@ void stack::call::answer(thread *thread, session *s)
 	}
 }
 
+void stack::call::message_reply(thread *thread, session *s)
+{
+	assert(thread != NULL);
+	assert(s != NULL);
+
+	int tid;
+	osip_message_t *reply = NULL;
+
+	Mutex::protect(this);
+	tid = s->tid;
+	Mutex::release(this);
+
+	if(tid < 1)
+		return;
+
+	eXosip_lock();
+	eXosip_call_send_answer(tid, thread->sevent->response->status_code, NULL);
+	eXosip_unlock();
+}
+
+
+void stack::call::call_reply(thread *thread, session *s)
+{
+	assert(thread != NULL);
+	assert(s != NULL);
+
+	int tid;
+	osip_body_t *body = NULL;
+	osip_message_t *reply = NULL;
+
+	Mutex::protect(this);
+	tid = s->tid;
+	Mutex::release(this);
+	if(tid < 1)
+		return;
+
+	eXosip_lock();
+	eXosip_call_build_answer(tid, thread->sevent->response->status_code, &reply);
+	if(reply) {
+		osip_message_set_require(reply, "100rel");
+		osip_message_set_header(reply, "RSeq", "1");
+		if(body && body->body) {
+			osip_message_set_body(reply, body->body, strlen(body->body));
+			osip_message_set_content_type(reply, "application/sdp");
+		}
+		eXosip_call_send_answer(tid, thread->sevent->response->status_code, reply);
+	}
+	eXosip_unlock();
+}
+
 void stack::call::confirm(thread *thread, session *s)
 {
 	assert(thread != NULL);
@@ -452,6 +612,7 @@ void stack::call::expired(void)
 
 	Mutex::protect(this);
 	switch(state) {
+	case TRANSFER:
 	case HOLDING:	// hold-recall timer expired...
 
 	case RINGING:	// re-generate ring event to origination...
@@ -460,6 +621,8 @@ void stack::call::expired(void)
 			reply_source(SIP_RINGING);
 			return;
 			
+	case REDIRECT:	// FIXME: add refer select of next segment if list....
+	
 	case RINGBACK:
 	case BUSY:		// invite expired
 	case JOINED:	// active call session expired without re-invite
@@ -512,5 +675,4 @@ void stack::call::log(void)
 	starting = 0l;
 }
 
-
 END_NAMESPACE
