@@ -24,6 +24,7 @@
 #include <pwd.h>
 #include <grp.h>
 #include <sys/time.h>
+#include <sys/resource.h>
 #include <limits.h>
 #include <errno.h>
 
@@ -501,7 +502,7 @@ static void command(const char *uid, const char *cmd, unsigned timeout)
 	sigaddset(&sigs, SIGALRM);
 	pthread_sigmask(SIG_BLOCK, &sigs, NULL);
 
-	config::utils(uid);
+	server::utils(uid);
 
 	if(!process::control(uid, "%d %s", getpid(), cmd)) {
 		fprintf(stderr, "*** sipw: %s; server not responding\n", cmd);
@@ -770,7 +771,7 @@ extern "C" int main(int argc, char **argv)
 		openlog("sipw", 0, LOG_USER);
 
 		if(!stricmp(*argv, "stop") || !stricmp(*argv, "reload") || !stricmp(*argv, "abort") || !stricmp(*argv, "restart")) {
-			config::utils(user);
+			server::utils(user);
 			if(!process::control(user, *argv)) {
 				fprintf(stderr, "*** sipw: %s; server not responding\n", *argv);
 				exit(2);
@@ -779,7 +780,7 @@ extern "C" int main(int argc, char **argv)
 		}
 
 		if(!stricmp(*argv, "check")) {
-			config::utils(user);
+			server::utils(user);
 			if(!process::control(user, *argv)) {
 				fprintf(stderr, "*** sipw: %s; server cannot be checked\n", *argv);
 				exit(2);
@@ -809,7 +810,7 @@ extern "C" int main(int argc, char **argv)
 				exit(-1);
 			}
 
-			config::utils(user);
+			server::utils(user);
 			printf("<!-- provision template example for realm %s -->\n", registry::getRealm());
 			printf("<provision>\n");
 			printf("  <user><id>%s</id>\n", userid);
@@ -830,6 +831,19 @@ extern "C" int main(int argc, char **argv)
 		if(!stricmp(*argv, "registry")) 
 			server::regdump();
 
+		if(!stricmp(*argv, "address")) {
+			if(!argv[1]) {
+				fprintf(stderr, "*** sipw: address: missing\n");
+				exit(-1);
+			}
+			if(argv[2]) {
+				fprintf(stderr, "*** sip: address: only one address\n");
+				exit(-1);
+			}
+			command(user, *argv, 30);
+		}
+
+
 		if(!stricmp(*argv, "state")) {
 			if(!argv[1]) {
 				fprintf(stderr, "*** sipw: state: selection missing\n");
@@ -904,12 +918,12 @@ extern "C" int main(int argc, char **argv)
 	else
 		foreground(user, cfgfile, priority);
 
-	config::reload(user);
+	server::reload(user);
 
 	if(restartable) 
 		restart();
 
-	config::startup();
+	server::startup();
 
 	if(dumping)
 		stack::enableDumping();
