@@ -386,11 +386,11 @@ bool registry::reload(service *cfg)
 			if(!stricmp(key, "mapped") && !isConfigured()) 
 				mapped_entries = atoi(value);
 			else if(!stricmp(key, "digest") && !isConfigured()) {
-				digest = strdup(value);
-				String::upper((char *)digest);
+				digest = value;
+				digest.upper();
 			}
 			else if(!stricmp(key, "realm") && !isConfigured())
-				realm = strdup(value);
+				realm = value;
 			else if(!stricmp(key, "prefix") && !isConfigured())
 				prefix = atoi(value);
 			else if(!stricmp(key, "range") && !isConfigured())
@@ -422,7 +422,7 @@ bool registry::reload(service *cfg)
 	memset(contacts, 0, sizeof(LinkedObject *) * keysize);
 	memset(publishing, 0, sizeof(LinkedObject *) * keysize);
 	memset(addresses, 0, sizeof(LinkedObject *) * keysize);
-	process::errlog(INFO, "realm %s", realm);
+	process::errlog(INFO, "realm %s", realm.c_str());
 	return true;
 }
 
@@ -502,7 +502,7 @@ registry::mapped *registry::create(const char *id)
 		return NULL;
 	}
 
-	node = config::getProvision(id);
+	node = server::getProvision(id);
 	cp = "none";
 	rr->type = MappedRegistry::EXPIRED;
 	rr->expires = 0;
@@ -526,7 +526,7 @@ registry::mapped *registry::create(const char *id)
 	else if(!stricmp(cp, "service"))
 		rr->type = MappedRegistry::SERVICE;
 	if(!node || rr->type == MappedRegistry::EXPIRED) {
-		config::release(node);
+		server::release(node);
 		if(listed && rr->inuse)
 			rr->type = MappedRegistry::TEMPORARY;
 		else if(listed) {
@@ -614,14 +614,14 @@ registry::mapped *registry::create(const char *id)
 		pro = NULL;
 		leaf = node->leaf("profile");
 		if(leaf)
-			pro = config::getProfile(leaf->getPointer());
+			pro = server::getProfile(leaf->getPointer());
 		if(!pro)
-			pro = config::getProfile("*");
+			pro = server::getProfile("*");
 		if(pro)
 			memcpy(&rr->profile, pro, sizeof(rr->profile));
 	}
 
-	config::release(node);
+	server::release(node);
 	rr->ext = 0;
 	rr->status = MappedRegistry::IDLE;
 
@@ -636,8 +636,6 @@ registry::mapped *registry::create(const char *id)
 			process::errlog(INFO, "releasing %s from extension %d", prior->userid, ext);
 			prior->ext = 0;
 		}
-		if(ext && extmap[ext - reg.prefix] != rr)
-			service::publish(NULL, "- map %d %d", ext, getIndex(rr));
 		extmap[ext - reg.prefix] = rr;
 		rr->ext = ext;
 		process::errlog(INFO, "activating %s; extension=%d", rr->userid, ext);
