@@ -151,8 +151,8 @@ private:
 
 	volatile time_t expires;
 
-	const char *digest;
-	const char *realm;
+	String digest;
+	String realm;
 	unsigned prefix;
 	unsigned range;
 	unsigned routes;
@@ -179,27 +179,28 @@ public:
 		{return reg.routes;};
 
 
-	__EXPORT static unsigned getEntries(void);
-	__EXPORT static unsigned getIndex(mapped *rr);
-	__EXPORT static bool isExtension(const char *id);
-	__EXPORT static bool isUserid(const char *id);
-	__EXPORT static mapped *address(struct sockaddr *addr);
-	__EXPORT static mapped *contact(const char *uri);
-	__EXPORT static mapped *contact(struct sockaddr *addr, const char *uid);
-	__EXPORT static mapped *getExtension(const char *id);
-	__EXPORT static mapped *create(const char *id);
-	__EXPORT static mapped *access(const char *id);
-	__EXPORT static mapped *invite(const char *id);
-	__EXPORT static bool exists(const char *id);
-	__EXPORT static pattern *getRouting(unsigned trs, const char *id);
-	__EXPORT static void detach(mapped *m);
-	__EXPORT static bool remove(const char *id);
-	__EXPORT static void cleanup(time_t period); 
+	static unsigned getEntries(void);
+	static unsigned getIndex(mapped *rr);
+	static bool isExtension(const char *id);
+	static bool isUserid(const char *id);
+	static mapped *address(struct sockaddr *addr);
+	static mapped *contact(const char *uri);
+	static mapped *contact(struct sockaddr *addr, const char *uid);
+	static mapped *getExtension(const char *id);
+	static mapped *create(const char *id);
+	static mapped *access(const char *id);
+	static mapped *invite(const char *id);
+	static bool exists(const char *id);
+	static pattern *getRouting(unsigned trs, const char *id);
+	static void detach(mapped *m);
+	static bool remove(const char *id);
+	static void cleanup(time_t period); 
 };
 
 class __LOCAL stack : private service::callback, private mapped_reuse<MappedCall>, public TimerQueue
 {
 private:
+	friend class proxy;
 	friend class thread;
 	friend class messages;
 
@@ -235,12 +236,12 @@ private:
 		time_t activates;
 		uint32_t sequence;
 		call *parent;
-		struct sockaddr_internet address;
-		struct sockaddr_internet iface;
 		time_t expires;					// session/invite expires...
 		time_t ringing;					// ring no-answer timer...
 
-		enum {OPEN, CLOSED, RING, BUSY, REORDER} state;
+		enum {OPEN, CLOSED, RING, BUSY, REORDER, REFER} state;
+
+		rtpproxy::session proxy;
 
 		char sdp[1024];					// sdp body to use in exchange
 		char identity[MAX_URI_SIZE];	// our effective contact/to point...
@@ -280,7 +281,7 @@ private:
 
 		enum {LOCAL, INCOMING, OUTGOING, REFER} type;
 
-		enum {INITIAL, TRYING, RINGING, RINGBACK, REORDER, HOLDING, ANSWERED, JOINED, BUSY, TERMINATE, FAILED, FINAL} state;
+		enum {INITIAL, TRYING, RINGING, RINGBACK, REORDER, HOLDING, ANSWERED, JOINED, TRANSFER, REDIRECT, BUSY, TERMINATE, FAILED, FINAL} state;
 
 		call();
 
@@ -289,6 +290,7 @@ private:
 		char dialed[MAX_IDENT_SIZE];	// user or ip address...
 		char subject[MAX_URI_SIZE];		// call subject
 		char refer[MAX_IDENT_SIZE];		// used in forward management
+		rtpproxy *rtp;
 
 		void reply_source(int error);
 		void decline(thread *thread);
@@ -296,6 +298,9 @@ private:
 		void busy(thread *thread, session *s = NULL);
 		void failed(thread *thread, session *s);
 		void answer(thread *thread, session *s);
+		void call_reply(thread *thread, session *s);
+		void message_reply(thread *thread, session *s);
+		void reinvite(thread *thread, session *s);
 		void trying(thread *thread);
 		void confirm(thread *thread, session *s);
 		void expired(void);
@@ -304,7 +309,7 @@ private:
 		void disconnectLocked(void);
 		void joinLocked(session *s);
 		void log(void);
-		void bye(session *s);
+		void bye(thread *thread, session *s);
 
 		OrderedIndex segments;
 		const char *reason;
@@ -335,8 +340,6 @@ private:
 	void update(void);
 	void modify(void);
 
-	static stack sip;
-
 	unsigned threading, priority;
 	size_t stacksize;
 
@@ -348,42 +351,45 @@ private:
 	const char *volatile published;
 	const char *volatile proxy;
 	const char *iface;
-	const char *agent;
-	const char *system;
-	const char *anon;
+	String agent;
+	String system;
+	String anon;
 	short port;
 	bool incoming, outgoing, dumping;
 	int send101;
-	int family, tlsmode, protocol;
 	timeout_t ring_timer, cfna_timer, reset_timer;
 	unsigned invite_expires;
 
+	int family, tlsmode, protocol;
+
+	static stack sip;
+
 public:
 	stack();
 
-	__EXPORT static const char *getScheme(void);
-	__EXPORT static void getInterface(struct sockaddr *iface, struct sockaddr *dest);
-	__EXPORT static session *create(int cid, int did, int tid);
-	__EXPORT static session *create(call *cr, int cid);
-	__EXPORT static void destroy(session *s);
-	__EXPORT static void destroy(call *cr);
-	__EXPORT static void disjoin(call *cr);
-	__EXPORT static void detach(session *s);
-	__EXPORT static void clear(session *s);
-	__EXPORT static void close(session *s);
-	__EXPORT static session *access(int cid);
-	__EXPORT static char *sipHostid(const char *uri, char *buf, size_t size);
-	__EXPORT static char *sipUserid(const char *uri, char *buf, size_t size);
-	__EXPORT static char *sipAddress(struct sockaddr_internet *addr, char *buf, const char *user = NULL, size_t size = MAX_URI_SIZE);
-	__EXPORT static char *sipPublish(struct sockaddr_internet *addr, char *buf, const char *user = NULL, size_t size = MAX_URI_SIZE);
-	__EXPORT static char *sipIdentity(struct sockaddr_internet *addr, char *buf, const char *user = NULL, size_t size = MAX_IDENT_SIZE);
-	__EXPORT static char *sipContact(struct sockaddr_internet *addr, char *buf, const char *user = NULL, const char *display = NULL, size_t size = MAX_URI_SIZE);
-	__EXPORT static Socket::address *getAddress(const char *uri, Socket::address *addr = NULL);
-	__EXPORT static void siplog(osip_message_t *msg);
-	__EXPORT static void enableDumping(void);
-	__EXPORT static void infomsg(session *session, eXosip_event_t *sevent);
-	__EXPORT static void setDialog(session *session, int did);
-	__EXPORT static int getDialog(session *session);
+	static const char *getScheme(void);
+	static void getInterface(struct sockaddr *iface, struct sockaddr *dest);
+	static session *create(int cid, int did, int tid);
+	static session *create(call *cr, int cid);
+	static void destroy(session *s);
+	static void destroy(call *cr);
+	static void disjoin(call *cr);
+	static void detach(session *s);
+	static void clear(session *s);
+	static void close(session *s);
+	static session *access(int cid);
+	static char *sipHostid(const char *uri, char *buf, size_t size);
+	static char *sipUserid(const char *uri, char *buf, size_t size);
+	static char *sipAddress(struct sockaddr_internet *addr, char *buf, const char *user = NULL, size_t size = MAX_URI_SIZE);
+	static char *sipPublish(struct sockaddr_internet *addr, char *buf, const char *user = NULL, size_t size = MAX_URI_SIZE);
+	static char *sipIdentity(struct sockaddr_internet *addr, char *buf, const char *user = NULL, size_t size = MAX_IDENT_SIZE);
+	static char *sipContact(struct sockaddr_internet *addr, char *buf, const char *user = NULL, const char *display = NULL, size_t size = MAX_URI_SIZE);
+	static Socket::address *getAddress(const char *uri, Socket::address *addr = NULL);
+	static void siplog(osip_message_t *msg);
+	static void enableDumping(void);
+	static void infomsg(session *session, eXosip_event_t *sevent);
+	static void setDialog(session *session, int did);
+	static int getDialog(session *session);
 
 	inline static timeout_t ringTimeout(void)
 		{return stack::sip.ring_timer;};
@@ -398,7 +404,7 @@ public:
 		{return stack::sip.invite_expires;};
 };
 
-class __LOCAL config : public service
+class __LOCAL server : public service
 {
 private:
 	typedef	linked_value<profile_t, LinkedObject> profile;
@@ -423,22 +429,31 @@ private:
 	void dump(FILE *fp);
 
 public:
-	config(const char *id);
+	server(const char *id);
 
-	__EXPORT static bool check(void);
-	__EXPORT static profile_t *getProfile(const char *id); 
-	__EXPORT static keynode *getRouting(const char *id);
-	__EXPORT static keynode *getProvision(const char *id);
-	__EXPORT static keynode *getExtension(const char *id);
-	__EXPORT static unsigned getForwarding(const char *id);
-	__EXPORT static cidr *getPolicy(struct sockaddr *addr);
-	__EXPORT static bool isLocal(struct sockaddr *addr);
-	__EXPORT static void release(cidr *access);
-	__EXPORT static void release(keynode *node);
-	__EXPORT static void reload(const char *uid);
-	__EXPORT static void utils(const char *uid);
-	__EXPORT static Socket::address *getContact(const char *id);
-	__EXPORT static unsigned forwarding(keynode *node);
+	static bool check(void);
+	static profile_t *getProfile(const char *id); 
+	static keynode *getRouting(const char *id);
+	static keynode *getProvision(const char *id);
+	static keynode *getExtension(const char *id);
+	static keynode *getConfig(void);
+	static unsigned getForwarding(const char *id);
+	static cidr *getPolicy(struct sockaddr *addr);
+	static bool isLocal(struct sockaddr *addr);
+	static void release(cidr *access);
+	static void release(keynode *node);
+	static void reload(const char *uid);
+	static void utils(const char *uid);
+	static Socket::address *getContact(const char *id);
+	static unsigned forwarding(keynode *node);
+	static void plugins(const char *argv0, const char *names);
+	static void usage(void);
+	static void version(void);
+	static void run(const char *user);
+	static void stop(void);
+	static caddr_t allocate(size_t size, LinkedObject **list, volatile unsigned *count = NULL);
+	static unsigned allocate(void);
+	static void regdump(void);	
 };
 
 class __LOCAL messages : public service::callback
@@ -507,6 +522,7 @@ private:
 	const char *via_host;
 	unsigned via_hops;
 	unsigned via_port;
+	rtpproxy::session proxyinfo;
 
 	enum {EXTERNAL, LOCAL, PUBLIC, ROUTED, FORWARDED} destination;
 	enum {CALL, MESSAGE, REGISTRAR, NONE} authorizing;
@@ -537,20 +553,8 @@ private:
 	const char *getIdent(void);
 
 public:
-	__EXPORT static void shutdown(void);
-};
-
-class __EXPORT server
-{
-public:
-	static void plugins(const char *argv0, const char *names);
-	static void usage(void);
-	static void version(void);
-	static void run(const char *user);
-	static void stop(void);
-	static caddr_t allocate(size_t size, LinkedObject **list, volatile unsigned *count = NULL);
-	static unsigned allocate(void);
-	static void regdump(void);
+	static void shutdown(void);
+	static bool assign(stack::call *cr, unsigned count);
 };
 
 END_NAMESPACE
