@@ -451,6 +451,7 @@ void stack::destroy(call *cr)
 	assert(cr != NULL);
 
 	linked_pointer<segment> sp;
+	rtpproxy *rtp = NULL;
 
 	cr->log();
 
@@ -476,11 +477,14 @@ void stack::destroy(call *cr)
 		delete *sp;
 		sp = next;
 	}
+	rtp = cr->rtp;
 	if(cr->map)
 		cr->map->enlist(&freemaps);
 	cr->delist();
 	delete cr;
 	locking.share();
+	if(rtp)
+		rtp->release();	
 }
 
 void stack::getInterface(struct sockaddr *iface, struct sockaddr *dest)
@@ -651,8 +655,6 @@ bool stack::reload(service *cfg)
 	linked_pointer<service::keynode> tp = cfg->getList("timers");
 	int val;
 	const char *localhosts = "localhost, localhost.localdomain";
-	const char *system_id = "sipwitch";
-	const char *anon_id = "anonymous";
 
 	unsigned cfna_value = 0;
 	unsigned ring_value = 0;
@@ -704,15 +706,15 @@ bool stack::reload(service *cfg)
 			else if(!stricmp(key, "trusted"))
 				trusted = cfg->dup(value);
 			else if(!stricmp(key, "system"))
-				system_id = cfg->dup(value);
+				system = value;
 			else if(!stricmp(key, "anon"))
-				anon_id = cfg->dup(value);
+				anon = value;
 			else if(!stricmp(key, "published") || !stricmp(key, "public"))
 				published = cfg->dup(value);
 			else if(!stricmp(key, "proxy") || !stricmp(key, "outbound"))
 				new_proxy = cfg->dup(value);
 			else if(!stricmp(key, "agent") && !isConfigured())
-				agent = strdup(value);
+				agent = value;
 			else if(!stricmp(key, "port") && !isConfigured())
 				port = atoi(value);
 			else if(!stricmp(key, "mapped") && !isConfigured())
@@ -743,11 +745,12 @@ bool stack::reload(service *cfg)
 		tp.next();
 	}
 
-	system = system_id;
-	anon = anon_id;
 	localnames = localhosts;
 	proxy = new_proxy;
 
+	if(family != AF_INET)
+		rtpproxy::enableIPV6();
+
 	if(ring_value && ring_value < 100)
 		ring_timer = ring_value * 1000l;
 	else if(ring_value >= 100)
