@@ -53,6 +53,18 @@ thread::thread() : DetachedThread(stack::sip.stacksize)
 	session = NULL;
 }
 
+bool thread::assign(stack::call *cr, unsigned count)
+{
+	if(cr->rtp)
+		return true;
+
+	cr->rtp = rtpproxy::create(count);
+	if(cr->rtp)
+		return true;
+
+	return false;
+}
+
 void thread::inviteRemote(stack::session *s, const char *uri_target)
 {
 	assert(s != NULL && s->parent != NULL);
@@ -79,7 +91,11 @@ void thread::inviteRemote(stack::session *s, const char *uri_target)
 	}
 	
 	snprintf(touri, sizeof(touri), "<%s>", uri_target);
+
+	proxyinfo.clear();
+	service::classify(&proxyinfo, &call->source->proxy, NULL);
 	invite = NULL;
+
 	eXosip_lock();
 	if(eXosip_call_build_initial_invite(&invite, touri, s->from, NULL, call->subject)) {
 		process::errlog(ERRLOG, "cannot invite %s; build failed", uri_target);
@@ -143,6 +159,7 @@ void thread::inviteRemote(stack::session *s, const char *uri_target)
 		
 	eXosip_unlock();
 	invited = stack::create(call, cid);
+	rtpproxy::copy(&invited->proxy, &proxyinfo);
 	stack::sipUserid(uri_target, username, sizeof(username));
 	stack::sipHostid(uri_target, route, sizeof(route));
 	String::set(invited->identity, sizeof(invited->identity), uri_target);
@@ -163,7 +180,6 @@ void thread::inviteRemote(stack::session *s, const char *uri_target)
 	}
 }
 
-
 void thread::inviteLocal(stack::session *s, registry::mapped *rr)
 {
 	assert(s != NULL && s->parent != NULL);
@@ -245,6 +261,13 @@ void thread::inviteLocal(stack::session *s, registry::mapped *rr)
 			goto next;
 		}
 
+		proxyinfo.clear();
+
+		// if proxy required, but not available, then we must skip this
+		// invite...
+		if(service::classify(&proxyinfo, &call->source->proxy, (struct sockaddr *)&tp->address) && !assign(call, 4))
+			goto next;
+
 		invite = NULL;
 		eXosip_lock();
 
@@ -337,7 +360,8 @@ void thread::inviteLocal(stack::session *s, registry::mapped *rr)
 		eXosip_unlock();
 
 		invited = stack::create(call, cid);
-
+		rtpproxy::copy(&invited->proxy, &proxyinfo);
+		
 		if(rr->ext) 
 			snprintf(invited->sysident, sizeof(invited->sysident), "%u", rr->ext);
 		else
@@ -454,6 +478,21 @@ void thread::invite(void)
 	osip_header_t *msgheader = NULL;
 	char fromext[32];
 
+	// FIXME: we should get proxy count extimate from sdp into global thread object...
+
+	// assign initial proxy if required to accept call...
+	// if no proxy available, then return 503...
+	if(service::classify(&session->proxy, &call->source->proxy, via_address.getAddr())) {
+		if(!assign(call, 4)) {
+noproxy:
+			send_reply(SIP_SERVICE_UNAVAILABLE);
+			call->failed(this, session);
+			return;
+		}
+	}
+	else if(destination == EXTERNAL && service::isProxied() && !assign(call, 4))
+		goto noproxy;
+		
 	if(extension)
 		snprintf(fromext, sizeof(fromext), "%u", extension);
 
@@ -491,7 +530,7 @@ void thread::invite(void)
 	else if(reginfo) {
 		target = reginfo->userid;
 		toext = reginfo->ext;
-		call->fwdmask = config::getForwarding(reginfo->userid);
+		call->fwdmask = server::getForwarding(reginfo->userid);
 		String::set(call->refer, sizeof(call->refer), reginfo->userid);
 	}
 
@@ -619,7 +658,7 @@ void thread::invite(void)
 	if(reginfo) {
 		// get rid of config ref if we are calling registry target
 		if(dialed) {
-			config::release(dialed);
+			server::release(dialed);
 			dialed = NULL;
 		}
 
@@ -661,7 +700,7 @@ void thread::identify(void)
 	String::set(display, sizeof(display), rr->display);
 	extension = rr->ext;
 	String::set(identity, sizeof(identity), rr->userid);
-	authorized = config::getProvision(identity);
+	authorized = server::getProvision(identity);
 	registry::detach(rr);
 }
 
@@ -696,7 +735,7 @@ bool thread::unauthenticated(void)
 	extension = rr->ext;
 	String::set(display, sizeof(display), rr->display);
 	String::set(identity, sizeof(identity), rr->userid);
-	authorized = config::getProvision(identity);
+	authorized = server::getProvision(identity);
 	registry::detach(rr);
 	if(authorized)
 		return true;
@@ -833,7 +872,7 @@ rewrite:
 	}
 
 	reginfo = registry::access(target);
-	dialed = config::getProvision(target);
+	dialed = server::getProvision(target);
 
 	debug(4, "rewrite process; registry=%p, dialed=%p\n", reginfo, dialed);
 
@@ -926,7 +965,7 @@ routing:
 		cp = service::getValue(authorized, "profile");
 		if(!cp)
 			cp = "*";
-		pro = config::getProfile(cp);
+		pro = server::getProfile(cp);
 		level = pro->level;
 	}
 	else
@@ -960,7 +999,7 @@ routing:
 	return true;
 
 static_routing:
-	dialed = config::getRouting(target);
+	dialed = server::getRouting(target);
 	if(!dialed)
 		goto invalid;
 
@@ -993,7 +1032,7 @@ static_routing:
 	if(!stricmp(dialed->getId(), "rewrite")) {
 		String::set(dbuf, sizeof(dbuf), dialing);
 		target = dbuf;
-		config::release(dialed);
+		server::release(dialed);
 		if(reginfo)
 			registry::detach(reginfo);
 		dialed = NULL;
@@ -1118,7 +1157,7 @@ bool thread::authenticate(void)
 		}
 	}
 
-	node = config::getProvision(auth->username);
+	node = server::getProvision(auth->username);
 	if(!node) {
 		process::errlog(NOTICE, "rejecting unknown %s", auth->username);
 		error = SIP_NOT_FOUND;
@@ -1177,7 +1216,7 @@ bool thread::authenticate(void)
 	return true;
 
 failed:
-	config::release(node);
+	server::release(node);
 	send_reply(error);
 	return false;
 }
@@ -1266,7 +1305,7 @@ bool thread::getsource(void)
         via_host = param->gvalue;
 		
 	via_address.set(via_host, via_port);
-	access = config::getPolicy(via_address.getAddr());
+	access = server::getPolicy(via_address.getAddr());
 	return true;
 }
 
@@ -1290,7 +1329,7 @@ void thread::validate(void)
 	remove_quotes(auth->nonce);
 	remove_quotes(auth->response);
 
-	node = config::getProvision(auth->username);
+	node = server::getProvision(auth->username);
 	if(!node) {
 		error = SIP_NOT_FOUND;
 		goto reply;
@@ -1338,7 +1377,7 @@ reply:
 	else
 		debug(2, "rejecting %s; error=%d", auth->username, error);
 
-	config::release(node);
+	server::release(node);
 	eXosip_lock();
 	eXosip_message_build_answer(sevent->tid, error, &reply);
 	if(reply != NULL) {
@@ -1569,7 +1608,7 @@ void thread::getDevice(registry::mapped *rr)
 {
 	assert(rr != NULL);
 
-	linked_pointer<service::keynode> device = config::list("devices");
+	linked_pointer<service::keynode> device = server::list("devices");
 
 	while(device) {
 		linked_pointer<service::keynode> node = device->getFirst();
@@ -1807,6 +1846,26 @@ void thread::run(void)
 				}
 			}
 			break;
+		case EXOSIP_CALL_REINVITE:
+			stack::siplog(sevent->request);
+			authorizing = CALL;
+			if(!sevent->request)
+				break;
+			if(sevent->cid < 1 && sevent->did < 1) {
+				send_reply(SIP_NOT_FOUND);
+				break;
+			}
+			expiration();
+			session = stack::access(sevent->cid);
+			if(!session) {
+				send_reply(SIP_NOT_FOUND);
+				break;
+			};
+			if(!authorize())
+				break;
+
+			session->parent->reinvite(this, session);
+			break;
 		case EXOSIP_CALL_INVITE:
 			stack::siplog(sevent->request);
 			authorizing = CALL;
@@ -1820,11 +1879,28 @@ void thread::run(void)
 				invite();
 			break;
 		case EXOSIP_CALL_MESSAGE_ANSWERED:
-			// probably got here as result of sending BYE to UA...
-		case EXOSIP_MESSAGE_ANSWERED:
-			// we already acknowledged SMS message when posting, so we
-			// throw this response away....
 			stack::siplog(sevent->response);
+			authorizing = CALL;
+			if(!sevent->response)
+				break;
+			if(sevent->cid < 1)
+				break;
+			session = stack::access(sevent->cid);
+			if(session)
+				session->parent->call_reply(this, session);
+			break;
+		case EXOSIP_MESSAGE_ANSWERED:
+			stack::siplog(sevent->response);
+			authorizing = MESSAGE;
+			if(!sevent->response)
+				break;
+			if(sevent->cid < 1)
+				break;
+			session = stack::access(sevent->cid);
+			if(session)
+				session->parent->message_reply(this, session);
+			else
+				send_reply(SIP_NOT_FOUND);
 			break;
 		case EXOSIP_CALL_MESSAGE_NEW:
 			stack::siplog(sevent->request);
@@ -1834,8 +1910,10 @@ void thread::run(void)
 					session = stack::access(sevent->cid);
 				if(session) {
 					send_reply(SIP_OK);
-					session->parent->bye(session);
+					session->parent->bye(this, session);
 				}
+				else
+					send_reply(SIP_NOT_FOUND);
 			}
 			break;
 		case EXOSIP_MESSAGE_NEW:
@@ -1904,17 +1982,17 @@ void thread::run(void)
 		// release config access lock(s)...
 
 		if(access) {
-			config::release(access);
+			server::release(access);
 			access = NULL;
 		}
 
 		if(dialed) {
-			config::release(dialed);
+			server::release(dialed);
 			dialed = NULL;
 		}
 
 		if(authorized) {
-			config::release(authorized);
+			server::release(authorized);
 			authorized = NULL;
 		}
 
